      subroutine angmom(jmol,nfrag,ifrag)
c
c   calculates the components and magnitude of the angular momentum
c   of the system of particles ifrag(i), i=1,...,nfrag given positions (x,y,z)
c   and velocities (vx,vy,vz) with respect to a desired origin.
c
      implicit real*8 (a-h,o-z)

      parameter(nmax=2000,natom=3)

      dimension ifrag(natom)

      common /cart1/ amas(natom),c(nmax,natom,3),cold(nmax,natom,3)

      common /cart2/ v(nmax,natom,3),vold(nmax,natom,3)

      common /iniq/ tc(1,natom,3),tccom(3),ccom(nmax,3),vcom(nmax,3),
     .  quanta(3*natom),eknm(3*natom),
     .  qnm(3*natom),vnm(3*natom),srmas(natom),srimas(natom)

      common /angpar/ rintns(3,3),rinvit(3,3),
     .  wang(3),amvec(3),ammag,angmo,nangmo

c
      amvec(1)=0.0d0
      amvec(2)=0.0d0
      amvec(3)=0.0d0
c
      do 1000 i=1,nfrag
        cxcom=c(jmol,ifrag(i),1)-ccom(jmol,1)
        cycom=c(jmol,ifrag(i),2)-ccom(jmol,2)
        czcom=c(jmol,ifrag(i),3)-ccom(jmol,3)
        vxcom=v(jmol,ifrag(i),1)-vcom(jmol,1)
        vycom=v(jmol,ifrag(i),2)-vcom(jmol,2)
        vzcom=v(jmol,ifrag(i),3)-vcom(jmol,3)

        amvec(1)=amvec(1)+(cycom*vzcom-czcom*vycom)*amas(ifrag(i))
        amvec(2)=amvec(2)+(czcom*vxcom-cxcom*vzcom)*amas(ifrag(i))
        amvec(3)=amvec(3)+(cxcom*vycom-cycom*vxcom)*amas(ifrag(i))

 1000 continue
      ammag=dsqrt(amvec(1)**2+amvec(2)**2+amvec(3)**2)
      return
      end

      subroutine angvel(iprint)
c
c  calculate angular velocity vector wang and inverse of moment
c  of inertia tensor rinvit from moment of inertia tensor rintns
c  and angular momentum vector amvec.
c       
      implicit real*8 (a-h,o-z)

      common /angpar/ rintns(3,3),rinvit(3,3),
     .  wang(3),amvec(3),ammag,angmo,nangmo

c
      do 20 i=1,3
        wang(i)=amvec(i)
        do 10 j=1,3
          rinvit(i,j)=rintns(i,j)
   10   continue
   20 continue
c
      call gaussj(rinvit,3,3,wang,1,1)
c
      if(iprint.gt.0)then
        write(11,100)
  100   format(/,' *** routine angvel: calculation of angular velocity')
c
       write(11,120)
  120   format(/,' moment of inertia tensor =')
        do 140 i=1,3
          write(11,130)(rintns(i,j),j=1,3)
  130     format(2x,3g12.5)
  140   continue
c
        write(11,150)
  150   format(/,' angular momentum vector =')
        write(11,130)(amvec(j),j=1,3)
c
       write(11,160)
  160   format(/,' inverse of moment of inertia tensor =')
        do 170 i=1,3
          write(11,130)(rinvit(i,j),j=1,3)
  170   continue
c
        write(11,180)
  180   format(/,' angular velocity vector =')
        write(11,130)(wang(j),j=1,3)
c
      endif
      return
      end

      subroutine velrot(jmol,nfrag,ifrag,vrot,iprint)
c
c  calculates velocities of rotation (vrot(iat,ixyz),ixyz=1,3) for each atom
c  given the position coordinates c(jmol,iat,ixyz) relative to the required 
c  origin (e.g. centre of mass) and the angular velocity vector wang.
c   vrot = wang (cross product) (x,y,z)
c        = | i     j     k     | = i [ w(2)*z(i)-w(3)*y(i) ]
c          | w(1)  w(2)  w(3)  | + j [ w(3)*x(i)-w(1)*z(i) ]
c          | x(i)  y(i)  z(i)  | + k [ w(1)*y(i)-w(2)*x(i) ]
c
      implicit real*8 (a-h,o-z)
      parameter(nmax=2000,natom=3)

      dimension vrot(natom,3),ifrag(natom)

      common /cart1/ amas(natom),c(nmax,natom,3),cold(nmax,natom,3)

      common /angpar/ rintns(3,3),rinvit(3,3),
     .  wang(3),amvec(3),ammag,angmo,nangmo


c
      do 50 i=1,nfrag
        vrot(ifrag(i),1) = wang(2)*c(jmol,ifrag(i),3)
     .    - wang(3)*c(jmol,ifrag(i),2)
        vrot(ifrag(i),2) = wang(3)*c(jmol,ifrag(i),1)
     .    - wang(1)*c(jmol,ifrag(i),3)
        vrot(ifrag(i),3) = wang(1)*c(jmol,ifrag(i),2)
     .    - wang(2)*c(jmol,ifrag(i),1)
   50 continue
c
      if(iprint.gt.0)then
        write(11,100)
  100   format(/,' *** routine velrot:',
     +  ' calculates velocities of rotation')
c
        write(11,120)
  120   format(/,' i,x(i),y(i),z(i) =')
        do 140 i=1,nfrag
          write(11,130)ifrag(i),c(jmol,ifrag(i),1),
     .      c(jmol,ifrag(i),2),c(jmol,ifrag(i),3)
  130     format(2x,i4,3g12.5)
  140   continue
c
        write(11,150)(wang(i),i=1,3)
  150   format(/,' angular velocity vector = (',3g12.5,' )')
c
        write(11,160)
  160   format(/,' i,vxrot(i),vyrot(i),vzrot(i) =')
        do 180 i=1,nfrag
          write(11,130)ifrag(i),vrot(ifrag(i),1),
     .      vrot(ifrag(i),2),vrot(ifrag(i),3)
  180   continue
c
      endif
      return
      end

      subroutine roten(erot,iprint)
c
c  calculates rotational energy erot from angular momentum vector amvec
c  and angular velocity vector wang
c       
      implicit real*8 (a-h,o-z)

      common /angpar/ rintns(3,3),rinvit(3,3),
     .  wang(3),amvec(3),ammag,angmo,nangmo
c
      erot=0.5d0*(wang(1)*amvec(1)+wang(2)*amvec(2)+wang(3)*amvec(3))
c
      if(iprint.gt.0)then
        write(11,100)(amvec(i),i=1,3),(wang(i),i=1,3),erot
  100   format(/,' routine roten: calculates rotational energy',
     +  /,' angular momentum vector = (',3g12.5,' )',
     +  /,' angular velocity vector = (',3g12.5,' )',
     +  /,' rotational energy =',g12.5)
      endif
      return
      end

