c
c  subroutine package calculates rotational constants for
c  a given input configuration
c  contains subroutines:
c                BVALUE,INTENS,JACOBI,EIGSRT
c
c  the other subroutines necessary for this calculation:
c                COPY12,CMZERC,CMZERV
c  are contained in misc.f
c
c****************************************************************

       subroutine bvalue(nfrag,ifrag,tmas)

       include 'traj.inc'

       dimension eval(3),vect(3,3)
       dimension ifrag(natom)

c  subroutine calculates moments of inertia and rotational
c  constants

       call lisym(11,'-')
       write(11,*)' *** subroutine bval:'
       write(11,*)

c  save initial positions 

      do 300 i=1,natom
        do 250 j=1,nmol
          cinit(j,i,1)=c(j,i,1)
          cinit(j,i,2)=c(j,i,2)
          cinit(j,i,3)=c(j,i,3)
  250 continue
  300 continue

c  move fragment to centre-of-mass coords 

      call cmzerc(nfrag,ifrag,tmas)

c   calculate the fragment's moment of inertia

       call intens(1,nmax,natom,nfrag,ifrag,c,amas,rintns,0)

c calculate eigenvectors of the inertia tensor rintns

       call jacobi(rintns,3,3,eval,vect,nrot)

c  sort normalised eigenvectors, vect, wrt descending eigenvalues, eval

       call eigsrt(eval,vect,3,3)

c  write out principal moments of inertia, eval

       write(11,*)'     ** NON-INFINITE ROTATIONAL CONSTANTS FOR ',
     &               'INPUT CONFIGURATION,  CM-1 **'
       write(11,*)
       if(eval(1).gt.1.0d-6) write(11,1) 16.8580d0/eval(1)
       if(eval(2).gt.1.0d-6) write(11,1) 16.8580d0/eval(2)
       if(eval(3).gt.1.0d-6) write(11,1) 16.8580d0/eval(3)
       write(11,*)

 1     format(f13.6)

c  restore original position 

      do 400 i=1,natom
        do 350 j=1,nmol
          c(j,i,1)=cinit(j,i,1)
          c(j,i,2)=cinit(j,i,2)
          c(j,i,3)=cinit(j,i,3)
  350 continue
  400 continue

      return
      end

c****************************************************************

      subroutine intens(jmol,nmax,natom,nfrag,ifrag,c,
     .  amas,rintns,iprint)
c
c  calculate elements in the moment of inertia tensor
c
c 891126:corrected Ixx, Iyy, Izz       
c
      implicit real*8 (a-h,o-z)
      dimension c(nmax,natom,3),amas(natom)
      dimension rintns(3,3)
      dimension ifrag(natom)
c
c     write(11,*)' *** subroutine intens'

c   initialise elements of inertia tensor
c
      do 200 i=1,3
        do 100 j=1,3
          rintns(i,j)=0.0d0
  100   continue
  200 continue

c
c   calculate elements
c
      do 1000 iat=1,nfrag
        rintns(1,1)=rintns(1,1)
     .    +amas(ifrag(iat))*(c(jmol,ifrag(iat),2)**2
     .    +c(jmol,ifrag(iat),3)**2)
        rintns(2,2)=rintns(2,2)
     .    +amas(ifrag(iat))*(c(jmol,ifrag(iat),3)**2
     .    +c(jmol,ifrag(iat),1)**2)
        rintns(3,3)=rintns(3,3)
     .    +amas(ifrag(iat))*(c(jmol,ifrag(iat),1)**2
     .    +c(jmol,ifrag(iat),2)**2)
        rintns(1,2)=rintns(1,2)
     .    -amas(ifrag(iat))*c(jmol,ifrag(iat),1)*c(jmol,ifrag(iat),2)
        rintns(1,3)=rintns(1,3)
     .    -amas(ifrag(iat))*c(jmol,ifrag(iat),1)*c(jmol,ifrag(iat),3)
        rintns(2,3)=rintns(2,3)
     .    -amas(ifrag(iat))*c(jmol,ifrag(iat),2)*c(jmol,ifrag(iat),3)
 1000 continue
      rintns(2,1)=rintns(1,2)
      rintns(3,1)=rintns(1,3)
      rintns(3,2)=rintns(2,3)
c
c   if iprint>0 write out elements of inertia tensor
c
      if(iprint.gt.0)then
        write(11,1500)
 1500   format(/,' routine intens: Elements of Inertia Tensor:')
        do 1600 i=1,3
          write(11,1550)(rintns(i,j),j=1,3)
 1550     format(/,1x,3g18.10e3)
 1600   continue
      endif
      return
      end     

c****************************************************************

      subroutine jacobi(a,n,np,d,v,nrot)

      parameter (nmax=100)
      implicit double precision(a-h,o-z)
      common /jacob1/ b(nmax),z(nmax)
      dimension a(np,np),d(np),v(np,np)


c**** computes all eigenvalues and eigenvectors of a real symmetric matrix
c**** a, which is of size n by n, stored in aphysical np by np array.
c**** On output, elements of a above the diagonal are destroyed. d reutrns
c**** the eigenvalues of a in its first n elements. v is a matrix with the
c**** same logical and physical dimensions as a, whose columns contain, 
c**** on output, the normalized eigenvectors of a. nrot returns the number
c**** of Jacobi rotations that were required.

c**** initialise to the identity matrix:

      do 12 ip=1,n
        do 11 iq=1,n
          v(ip,iq)=0.0d0
11      continue
        v(ip,ip)=1.0d0
12    continue

c**** initialize b and d to the diagonal of a
c**** vector z will acculmulate terms of the form t*a(p,q)

      do 13 ip=1,n
        b(ip)=a(ip,ip)
        d(ip)=b(ip)
        z(ip)=0.0d0
13    continue

c**** set counter on number of Jacobi rotations

      nrot=0

      do 24 i=1,50

c**** set convergence limit at 1.0d-16

        sm=1.0d-16

c**** sum off-diagonal elements

        do 15 ip=1,n-1
          do 14 iq=ip+1,n
            sm=sm+abs(a(ip,iq))
14        continue
15      continue

c**** check for convergence

        if(sm.eq.0.0d0)return

        if(i.lt.4)then
          tresh=0.2d0*sm/n**2
        else
          tresh=0.0d0
        endif

        do 22 ip=1,n-1
          do 21 iq=ip+1,n
            g=100.0d0*abs(a(ip,iq))

c**** after 4 sweeps, skip the rotation if the off-diagonal element
c**** is small

            if((i.gt.4).and.(abs(d(ip))+g.eq.abs(d(ip)))
     *         .and.(abs(d(iq))+g.eq.abs(d(iq))))then
              a(ip,iq)=0.0d0
            else if(abs(a(ip,iq)).gt.tresh)then
              h=d(iq)-d(ip)
              if(abs(h)+g.eq.abs(h))then
                t=a(ip,iq)/h
              else
                theta=0.5d0*h/a(ip,iq)
                t=1.0d0/(abs(theta)+sqrt(1.0d0+theta**2))
                if(theta.lt.0.0d0)t=-t
              endif
              c=1.0d0/sqrt(1.0d0+t**2)
              s=t*c
              tau=s/(1.0d0+c)
              h=t*a(ip,iq)
              z(ip)=z(ip)-h
              z(iq)=z(iq)+h
              d(ip)=d(ip)-h
              d(iq)=d(iq)+h
              a(ip,iq)=0.0d0

c**** case for rotation 1 <= j < p

              do 16 j=1,ip-1
                g=a(j,ip)
                h=a(j,iq)
                a(j,ip)=g-s*(h+g*tau)
                a(j,iq)=h+s*(g-h*tau)
16            continue

c**** case for rotation p < j < q

              do 17 j=ip+1,iq-1
                g=a(ip,j)
                h=a(j,iq)
                a(ip,j)=g-s*(h+g*tau)
                a(j,iq)=h+s*(g-h*tau)
17            continue

c**** case for rotation q < j <= n

              do 18 j=iq+1,n
                g=a(ip,j)
                h=a(iq,j)
                a(ip,j)=g-s*(h+g*tau)
                a(iq,j)=h+s*(g-h*tau)
18            continue
              do 19 j=1,n
                g=v(j,ip)
                h=v(j,iq)
                v(j,ip)=g-s*(h+g*tau)
                v(j,iq)=h+s*(g-h*tau)
19            continue
              nrot=nrot+1
            endif
21        continue
22      continue

c**** update d with the sum of t*a(p,q) and reinitialize z

        do 23 ip=1,n
          b(ip)=b(ip)+z(ip)
          d(ip)=b(ip)
          z(ip)=0.0d0
23      continue
24    continue

      if(nrot.ge.50)then
        write(11,*)'50 iterations should never happen'
        write(11,*) '**** abort procedure in subroutine jacobi'
      endif

      return
      end

c****************************************************************

      subroutine eigsrt(d,v,n,np)

c**** given the eigenvalues d and eigenvectors v as output from jacobi
c**** this routine sorts the eigenvalues into descending order, and 
c**** rearranges the columns of v correspondingly. 
c**** the method is straight insertion.

      implicit double precision(a-h,o-z)
      dimension d(np),v(np,np)

      do 13 i=1,n-1
        k=i
        p=d(i)
        do 11 j=i+1,n
          if(d(j).ge.p)then
            k=j
            p=d(j)
          endif
11      continue
        if(k.ne.i)then
          d(k)=d(i)
          d(i)=p
          do 12 j=1,n
            p=v(j,i)
            v(j,i)=v(j,k)
            v(j,k)=p
12        continue
        endif
13    continue
      return
      end

c****************************************************************

