c
c  subroutine package contains control routines for the
c  trajectory calculations:
c                     CNTL,READZ,READC
c
c  (the ems control routine, emspar, is contained in emspac)
c
c  the user defined routines for the potential energy surface:
c                     CONSTANTS,DVDI
c
c  and the user defined end-test and output routines:
c                     OUTP,GWRITE
c  
c*******************************************************

       subroutine cntl

c   this subroutine reads in run control parameters from file on unit 5

      include 'traj.inc'
      include 'ems.inc'

c     call lisym(11,'-')
c     write(11,*) ' *** subroutine cntl:'
c    .  ,' read in run control parameters from unit 7 ***'

c     open (unit=7,file ='CNT',status='old')

      n3=3*natom
c---------------------------------------------------------------------
c
c   calculate and scale model parameters
c   units for inputing  masses,energies and lengths:
c   if (nunits) then (mass  energy   length     scale factor) are:
c         1           g/mol kcal/mol angstroms  2.045482828d13
c         2           g/mol kj/mol   angstroms  1.0d13
c         3           g/mol cm-1     angstroms  1.093739383d12
c         4           g/mol eV       angstroms  9.822693571d13
c         5           g/mol hartrees bohr       9.682886569d14
c         6           g/mol aJ       angstroms  9.124087591d13
c
c on output masses, energies, lengths and times are scaled by
c amsc, esc, rsc and tsc (in seconds), respectively and
c must be multiplied by these scale factors to be converted
c to the stated units.

      nunits=4
      amsc=1.0d0
      esc=1.0d0
      rsc=1.0d0
      scalef=1.d0
      if(nunits.eq.1)then
        scalef=2.045482828d13
        msunit=' g/mol'
        enunit=' kcal/mol'
        lnunit=' angstroms'
        eukjpm=4.184d0
      elseif(nunits.eq.2)then
        scalef=1.0d13
        msunit=' g/mol'
        enunit=' kJ/mol'
        lnunit=' angstroms'
        eukjpm=1.00d0
      elseif(nunits.eq.3)then
        scalef=1.093739383d12
        msunit=' g/mol'
        enunit=' cm-1'
        lnunit=' angstroms'
        eukjpm=1.196265839d-02
      elseif(nunits.eq.4)then
        scalef=9.822693571d13
        msunit=' g/mol'
        enunit=' eV'
        lnunit=' angstroms'
        eukjpm=96.48530899d0
      elseif(nunits.eq.5)then
        scalef=9.682886569d14
        msunit=' g/mol'
        enunit=' hartree'
        lnunit=' bohr'
        eukjpm=2625.499964d0
      elseif(nunits.eq.6)then
        scalef=9.124087591d13
        msunit=' g/mol'
        enunit=' 0.1382aJ'
        lnunit=' angstroms'
        eukjpm=83.24897441d0
      endif
      tsc=dsqrt(amsc/esc)*rsc/scalef
      tups=tsc*1.0d12

c   Avagadro's number

      avanum=6.0221367d23

c   speed of light in cm s-1 and cm tu-1

      ccmps=2.99792458d10
      ccmptu=ccmps*tsc

c   Planck's constant in J s, kJ/mol s and eu tu

      hjs=6.6260755d-34
      hkjpms=hjs*avanum*1.0d-03
      heutu=(hkjpms/tsc)/eukjpm

c---------------------------------------------------------------------
c   echo some parameters

c     write(11,40)nunits,esc,enunit,rsc,lnunit,amsc,msunit,
c    .  tsc,tups,scalef
   40 format(
     ./,'  unit scaling parameters',
     ./,'  nunits=',i3,
     ./,'  esc=',g12.5,a10,'  rsc=',g12.5,a10,'  amsc=',g12.5,a10,
     ./,'  tsc=',g17.10,' s =',g17.10,' ps  scalef=',g17.10,' s-1')

c     write(11,45)avanum,ccmps,ccmptu,hjs,hkjpms,heutu,eukjpm
   45 format(
     ./,'  Avagadro''s number   =',g17.10,
     ./,'  Speed of light       =',g17.10,' cm s-1  =',
     .g17.10,' cm tu-1',
     ./,'  Planck''s constant   =',g17.10,' J s     =',
     .g17.10,' kJ/mol s',
     ./,'                                           =',g17.10,' eu tu',
     ./,'  1 eu = 1 energy unit = ',g17.10,' kJ/mol')

c     call lisym(11,'-')

c   read in title of run
 
      write(6,*)' *** Program traj.hnc *** '
      write(6,*)
      write(6,*)' Enter title:'
      read(5,80) title
      write(6,*)
   80 format(a80)
      call lisym(11,'-')
      write(11,81) title
   81 format(1x,a79)
      call lisym(11,'-')
 
c   read in time step,no. of traj. step, nprint,no. of molecule etc.

      write(6,*)' enter delta = timestep (approximate units are fs)'
      write(6,*)' suggested value is 0.10'
      read(5,*)delta
      write(6,*)' chosen delta =',delta
      delta=delta*0.10d0
      write(6,*)' enter nstep = maximum number of trajectory steps'
      write(6,*)' suggested value is 20000'
      read(5,*)nstep
      write(6,*)' chosen nstep =',nstep
      if(nstep.gt.40000)then
        write(6,*)
        write(6,*)' *** ABORT ***'
        write(6,*)' maximum number of steps is 40,000 '
        stop
      endif
      write(6,*)' enter ntraj = total number of trajectories performed'
      read(5,*)nmol
      write(6,*)' chosen ntraj =',nmol 
      if(nmol.gt.nmax)then
        write(6,*)
        write(6,*)' *** ABORT ***'
        write(6,*)' maximum number of trajectories is 200 '
        stop
      endif
      nprint=100
      if(nmol.lt.4.and.nstep.gt.1)then
        write(6,*)' enter nprint = number of steps between ',
     &            'trajectory analysis'
        write(6,*)' suggested value is 200'
        read(5,*)nprint
      write(6,*)' chosen nprint =',nprint
      endif
c     read(7,80) comlin
c     write(11,81) comlin
c     read(7,*) delta,nstep,nprint,nmol
c     write(11,100) delta,nstep,nprint,nmol
  100 format(1x,g18.10,3i10)

      if(nstep.gt.1)then
        write(6,*)' enter random number seed for initial conditions'
        write(6,*)' for example 123456789' 
        read(5,*)ir
        write(6,*)' chosen ir =',ir
      endif
c     read(7,80) comlin
c     write(11,81) comlin
c     read(7,*) ninit,ir
c     write(11,120) ninit,ir
  120 format(1x,2i10)

c     read(7,80) comlin
c     write(11,81) comlin
c     read(7,*) dx,iprint
c     write(11,100) dx,iprint

c     read(7,80) comlin
c     write(11,81) comlin
c     read(7,*) efraga

c  hardwire this number

      efraga = 2.5d0

c     write(11,140) efraga
  140 format(1x,3g18.10)

c calculate pi, conversion factors
 
      pi=2.d0*dacos(0.d0)
      todeg=45.d0/atan(1.d0)
      toang=0.52917706d0
      sq2=dsqrt(2.d0)

c     write(11,*) ' conversion factors: '
c     write(11,*) ' todeg=',todeg,' toang=',toang
c     write(11,*) ' pi   =',pi,' sq2  =',sq2


c     close(unit=7)

      return
      end

c  -------------------------------------------------------------------

      subroutine readc
 
c   read in the cartesian coordinates and atomic masses of the
c   atoms before initial displacement from file on unit 7 
 
      include 'traj.inc'
 
c     call lisym(11,'-')
c     write(11,*) ' *** subroutine readc:',
c    .  ' read in atomic masses and cartesian coordinates',
c    .  ' from unit 7 ***'
c     write(11,*) ' '
 
c     open (unit=7,file ='CCM',status='old')

      n3=3*natom
      n33=n3*(n3+1)/2
      nvib=n3-6

c   read comment
 
c     read(7,80)comlin
   80 format(a80)
c     write(11,81)comlin
   81 format(1x,a79)

c   read in the atomic masses

c     write(11,*) ' atomic masses :'
c     read(7,*)(amas(i),i=1,natom)

c  hard wire these numbers:

      amas(1)=1.00783d0
      amas(2)=12.01115d0
      amas(3)=14.0067d0

c     write(11,100)(amas(i),i=1,natom)
  100 format(1x,4g18.10)
 
c   read in cartesian coords for one molecule :
 
      write(6,*)
      write(6,*)' Enter Cartesian coordinates for the ',
     &          'reference geometry:'
      write(6,*)'   This geometry represents the starting point for '
      write(6,*)'   the trajectory calculations'
      write(6,*)
      write(6,*)' coordinates for H atom (x,y,z) in Angstrom:'
      read(5,*)(tc(1,1,k),k=1,3)
      write(6,*)' coordinates for C atom (x,y,z) in Angstrom:'
      read(5,*)(tc(1,2,k),k=1,3)
      write(6,*)' coordinates for N atom (x,y,z) in Angstrom:'
      read(5,*)(tc(1,3,k),k=1,3)

c     read(7,80)comlin
c     write(11,81)comlin
c     read(7,*)((tc(1,i,k),k=1,3),i=1,natom)
      write(11,*)
      write(11,*)' REFERENCE CONFIGURATION:'
      write(11,*)
      do 120 i=1,natom
        write(11,110)(tc(1,i,k),k=1,3)
  110 format(1x,3g24.16)
  120 continue
      write(11,*)
 
c  scale atomic masses

      do 150 i=1,natom
        amas(i)=amas(i)/amsc
  150 continue

c  convert into scaled units
 
c  set all molecule coords initially the same and convert them 
c  into scaled units

      do 200 k=1,3
        do 200 i=1,natom
          tc(1,i,k)=tc(1,i,k)/rsc
          do 200 j=1,nmol
            c(j,i,k)=tc(1,i,k)
  200 continue

c     close(unit=7)

      return
      end

c  --------------------------------------------------------------

      subroutine readz

c   read the mb,nb,etc. for the 'z matrix' definition
c   of the molecule connectivity

      include 'traj.inc'
c
c     call lisym(11,'-')
c     write(11,*) ' *** subroutine readz:',
c    .  ' read in the mb,nb,etc. for the z matrix definition',
c    .  ' of the molecule connectivity from unit 7 ***'
c     write(11,*) ' '
 
c     open (unit=7,file='ZMA',status='old')

c   read comment
 
c     read(7,80)comlin
   80 format(a80)
c     write(11,81)comlin
   81 format(1x,a79)

c   read bonded atom numbers
 
c  hardwire thesi options

       mb(1)=1
       nb(1)=2

       mb(2)=2
       nb(2)=3

       mb(3)=1
       nb(3)=3

c     do 100 i=1,nbond
c       read(7,*) mb(i),nb(i)
c       write(11,*)mb(i),nb(i),'   - bond',i
c 100 continue

      if(nangle.eq.0) go to 1000

c   read comment
 
c     read(7,80)comlin
c     write(11,81)comlin

c   read atom numbers for angles
 
      do 200 i=1,nangle
c       read(7,*)la(i),ma(i),na(i)
c       write(11,*)la(i),ma(i),na(i)
  200 continue

c   read comment

c     read(7,80)comlin
c     write(11,81)comlin

c   read bond numbers for each angle
 
      do 400 i=1,nangle
c       read(7,*)lab(i),mab(i)
c       write(11,*)lab(i),mab(i)
  400 continue
 1000 continue

       if(ndihed.eq.0) go to 2000
 
c   read comment

c     read(7,80)comlin
c     write(11,81)comlin

c   read the atom numbers for the dihedrals
 
      do 1500 i=1,ndihed
c       read(7,*)kd(i),ld(i),md(i),nd(i)
c       write(11,*)kd(i),ld(i),md(i),nd(i)
 1500 continue

c   read comment about dihedral bond numbers
 
c     read(7,80)comlin
c     write(11,81)comlin

c   read dihedral bond numbers
 
      do 1600 i=1,ndihed
c       read(7,*)lda(i),ldb(i),ldc(i)
c       write(11,*)lda(i),ldb(i),ldc(i)
 1600 continue

c   read comment about dihedral angle numbers
 
c     read(7,80)comlin
c     write(11,81)comlin

c   read dihedral angle numbers
 
      do 1700 i=1,ndihed
c       read(7,*)kan(i),lan(i)
c       write(11,*)kan(i),lan(i)
 1700 continue

 2000 continue
      
c   read fragment atom numbers

c  hardwire these options

       nfraga=3
       nfragb=0

c     read(7,80)comlin
c     write(11,81)comlin

c     read(7,*)nfraga,nfragb
c     write(11,*)nfraga,nfragb

      if(nfraga+nfragb.ne.natom)then
c       write(11,*) 'ABORT: nfraga+nfragb.ne.natom'
c       write(11,*) nfraga,'+',nfragb,'.ne.',natom
        stop
      endif

c     read(7,80)comlin
c     write(11,81)comlin

      ifraga(1)=1
      ifraga(2)=2
      ifraga(3)=3

c     read(7,*)(ifraga(i),i=1,nfraga)
c     write(11,*)(ifraga(i),i=1,nfraga)

c     read(7,80)comlin
c     write(11,81)comlin

c     read(7,*)(ifragb(i),i=1,nfragb)
c     write(11,*)(ifragb(i),i=1,nfragb)

      tamas=0.0d0
      do 3700 n=1,nfraga
        tamas=tamas+amas(ifraga(n))
3700  continue
c     write(11,*)' mass of fragment A is:',tamas

      if (nfraga .le. 1) go to 4100

      nbfraga=0
      do 4000 i=1,nbond
	do 3900 j=1,nfraga
	  do 3800 k=1,nfraga
            if ((mb(i).eq.ifraga(j)).and.(nb(i).eq.ifraga(k))) then
              nbfraga=nbfraga+1
              ibfraga(nbfraga)=i
            endif
 3800     continue 
 3900   continue
 4000 continue
	
c     write(11,*) 'no. of bonds in fragment a is', nbfraga
c     write(11,*) 'bonds in fragment a are'
c     write(11,*) (ibfraga(i),i=1,nbfraga)

 4100 continue

      tbmas=0.0d0
      do 4700 n=1,nfragb
        tbmas=tbmas+amas(ifragb(n))
4700  continue
c     write(11,*)' mass of fragment B is:',tbmas

      if (nfragb .le. 1) go to 5100

      nbfragb=0
      do 5000 i=1,nbond
	do 4900 j=1,nfragb
	  do 4800 k=1,nfragb
            if ((mb(i).eq.ifragb(j)).and.(nb(i).eq.ifragb(k))) then
              nbfragb=nbfragb+1
              ibfragb(nbfragb)=i
              endif
 4800     continue 
 4900   continue
 5000 continue

c     write(11,*) 'no. of bonds in fragment b is', nbfragb
c     write(11,*) 'bonds in fragment b are'
c     write(11,*) (ibfragb(i),i=1,nbfragb)
      
 5100 continue

c     close(unit=7)

      return
      end

c*******************************************************************

       subroutine constants

       include 'traj.inc'

c  subroutine contains the necessary constants for the 
c  HCN <-> HNC ground state potential energy surface

c  potential taken from Murrell, Carter and Halonen, J. Mol. Spectr.
c  1982, Vol 93, page 307

c  original units are Angs-1, Angs-2, Angs-3, Angs and eV respectively 

c  two-body potential parameters:

c  CH (quartet sigma minus)

       aa1(1)=5.5297d0
       aa2(1)=8.7166d0
       aa3(1)=5.3082d0
       re(1)=1.0823d0
       de(1)=2.8521d0

c  CN (doublet sigma plus)

       aa1(2)=5.2448d0
       aa2(2)=7.3416d0
       aa3(2)=4.9785d0
       re(2)=1.1718d0
       de(2)=7.9282d0

c  NH (triplet sigma minus)

       aa1(3)=3.0704d0
       aa2(3)=0.0000d0
       aa3(3)=0.0000d0
       re(3)=1.0370d0
       de(3)=3.9938d0

c three body potential:

c  set the reference potential (eV):

       v0=-3.0578d0

c  reference bondlengths (Angs):

       r0(1)=1.9607d0
       r0(2)=2.2794d0
       r0(3)=1.8687d0

c  switching coefficients (Angs-1):

       gamma(1)=3.9742d0
       gamma(2)=4.3688d0
       gamma(3)=1.5176d0

c  polynomial coefficients


c  zero all the coefficients first (we will only use the "upper triangle"):

       do 100 i=1,3
         c1(i)=0.0d0
         do 100 j=1,3
           c2(i,j)=0.0d0
           do 100 k=1,3
             c3(i,j,k)=0.0d0
             do 100 l=1,3
               c4(i,j,k,l)=0.0d0
               do 100 m=1,3
                 c5(i,j,k,l,m)=0.0d0
100    continue

c  linear terms (Angs-1):

       c1(1)= 1.9076d0
       c1(2)=-0.5008d0
       c1(3)=-0.0149d0

c  second order (Angs-2):
 
       c2(1,1)= 0.6695d0
       c2(2,2)=-1.3535d0
       c2(3,3)=-1.0501d0
       c2(1,2)= 0.2698d0
       c2(1,3)=-1.1120d0
       c2(2,3)= 1.9310d0

c  third order (Angs-3):

       c3(1,1,1)=-0.0877d0
       c3(2,2,2)= 0.0044d0
       c3(3,3,3)= 0.0700d0
       c3(1,1,2)= 0.0898d0
       c3(1,2,2)=-1.0186d0
       c3(1,1,3)=-0.0911d0
       c3(1,3,3)= 0.0017d0
       c3(2,2,3)= 0.4567d0
       c3(2,3,3)=-0.8840d0
       c3(1,2,3)= 0.3333d0
    
c  fourth order (Angs-4):

       c4(1,1,1,1)=-0.0367d0
       c4(2,2,2,2)= 0.4821d0
       c4(3,3,3,3)= 0.2564d0
       c4(1,1,1,2)=-0.0017d0
       c4(1,1,2,2)=-0.2278d0
       c4(1,2,2,2)=-0.1287d0
       c4(1,1,1,3)= 0.1759d0
       c4(1,1,3,3)=-0.0399d0
       c4(1,3,3,3)=-0.1447d0
       c4(2,2,2,3)=-0.3147d0
       c4(2,2,3,3)= 0.1233d0
       c4(2,3,3,3)= 0.3161d0
       c4(1,1,2,3)= 0.0919d0
       c4(1,2,2,3)=-0.0954d0
       c4(1,2,3,3)= 0.1778d0

c  fifth order (Angs-5):

       c5(2,2,2,2,2)=-0.1892d0

c  define the rotation matrix:

       rot(1,1)= 0.4436d0
       rot(1,2)= 0.6091d0
       rot(1,3)= 0.6575d0
       rot(2,1)=-0.8941d0
       rot(2,2)= 0.2498d0
       rot(2,3)= 0.3718d0
       rot(3,1)= 0.0622d0
       rot(3,2)=-0.7527d0
       rot(3,3)= 0.6554d0

       return
       end

c*******************************************************************

      subroutine dvdi(j)

c  subroutine calculates the potential energy en and its internal derivatives
c  for the HCN <-> HNC ground state system

c  potential taken from Murrell, Carter and Halonen, J. Mol. Spectr.
c  1982, Vol 93, page 307

c  the coordinates used are R1 = CH distance
c                           R2 = CN distance 
c                           R3 = NH distance
c  ie 3 bondlengths

       include 'traj.inc'

       real*8 poly(nmax),switch(nmax)
       real*8 dre(nmax,3),rho(nmax,3),s(nmax,3)
       real*8 dsdr(nmax,3,3),dpdr(nmax,3),dswdr(nmax,3)

c  first zero various derivatives 

       do 100 i=1,3
         dvr(j,i)=0.0d0
         dpdr(j,i)=0.0d0
         dsdr(j,i,1)=0.0d0
         dsdr(j,i,2)=0.0d0
         dsdr(j,i,3)=0.0d0
         dswdr(j,i)=0.0d0

c  and calculate bond differences from their equilibrium and 
c  reference values 

         dre(j,i)=r(j,i)-re(i)
         rho(j,i)=r(j,i)-r0(i)
100    continue

c  now zero the energy 

         en(j)=0.0d0 
         poly(j)=0.0d0 
         switch(j)=1.0d0 

c  define the rotated coordinates s: 

         do 200 i=1,3
           s(j,i)=rot(i,1)*rho(j,1)+rot(i,2)*rho(j,2)+rot(i,3)*rho(j,3)
           
c  and calculate derivatives of s wrt the bond lengths r:

           do 200 k=1,3
             dsdr(j,i,k)= rot(i,k)
200    continue

c  calculate the potential following Murrell, Carter and Halonen as a
c  combination of 2- and 3-body terms 

c  first the 2-body terms:

       do 300 i=1,3
         en(j)=en(j) 
     &         -de(i)*(1.0d0+aa1(i)*dre(j,i)+aa2(i)*dre(j,i)**2 
     &                 +aa3(i)*dre(j,i)**3)*dexp(-aa1(i)*dre(j,i))
         dvr(j,i)=dvr(j,i)
     &         +aa1(i)*de(i)*(1.0d0+aa1(i)*dre(j,i)+aa2(i)*dre(j,i)**2
     &                       +aa3(i)*dre(j,i)**3)*dexp(-aa1(i)*dre(j,i))
     &         -de(i)*(aa1(i)+2.0d0*aa2(i)*dre(j,i)
     &                 +3.0d0*aa3(i)*dre(j,i)**2)*dexp(-aa1(i)*dre(j,i))
300    continue

c  and now the 3-body term:

c  calculate the polynomial contribution to the three-body potential:
c  NB the polynomial calculation is *NOT* set up to be vectorised,
c  ie the loop over nmol is on the very outside


c  zeroth order term:

         poly(j)=poly(j)+1.0d0

c  and next fifth order (since only a single fifth order term is present):

         poly(j)=poly(j)+c5(2,2,2,2,2)*s(j,2)**5

c  derivatives of polynomial wrt bondlength r:

         do 500 i1=1,3
            dpdr(j,i1)=dpdr(j,i1)
     &                   +c5(2,2,2,2,2)*dsdr(j,2,i1)*5.0d0*s(j,2)**4
 500      continue

c  now go back to the linear term:

         do 1000 i=1,3
           poly(j)=poly(j)+c1(i)*s(j,i)

c  derivative wrt bondlength r:

           do 600 i1=1,3
             dpdr(j,i1)=dpdr(j,i1)
     &                   +c1(i)*dsdr(j,i,i1)
 600        continue
         
c  second order:

           do 1000 k=i,3
             poly(j)=poly(j)+c2(i,k)*s(j,i)*s(j,k)

c  derivative wrt bondlength r:

             do 700 i1=1,3
               dpdr(j,i1)=dpdr(j,i1)
     &                     +c2(i,k)*dsdr(j,i,i1)*s(j,k)
     &                     +c2(i,k)*s(j,i)*dsdr(j,k,i1)
 700          continue 

c  third order: 

             do 1000 l=k,3 
               poly(j)=poly(j)+c3(i,k,l)*s(j,i)*s(j,k)*s(j,l) 

c  derivative wrt bondlength r: 

               do 800 i1=1,3 
                 dpdr(j,i1)=dpdr(j,i1)
     &                       +c3(i,k,l)*dsdr(j,i,i1)*s(j,k)*s(j,l)
     &                       +c3(i,k,l)*s(j,i)*dsdr(j,k,i1)*s(j,l)
     &                       +c3(i,k,l)*s(j,i)*s(j,k)*dsdr(j,l,i1)
 800          continue

c  fourth order:

             do 1000 m=l,3
                 poly(j)=poly(j)+c4(i,k,l,m)*s(j,i)*s(j,k)*s(j,l)*s(j,m)

c  derivative wrt bondlength r:

                 do 900 i1=1,3
                   dpdr(j,i1)=dpdr(j,i1)
     &                   +c4(i,k,l,m)*dsdr(j,i,i1)*s(j,k)*s(j,l)*s(j,m)
     &                   +c4(i,k,l,m)*s(j,i)*dsdr(j,k,i1)*s(j,l)*s(j,m)
     &                   +c4(i,k,l,m)*s(j,i)*s(j,k)*dsdr(j,l,i1)*s(j,m)
     &                   +c4(i,k,l,m)*s(j,i)*s(j,k)*s(j,l)*dsdr(j,m,i1)
 900              continue

 1000   continue
       
c  calculate switching function, ie the three body term must tend
c  to zero as any of the 3 bondlengths becomes large:

c  again this loop is *NOT* vectorised

         do 1100 i=1,3
           switch(j)=switch(j)*(1.0d0-dtanh(0.5d0*gamma(i)*rho(j,i)))
1100     continue

c  now calculate the total energy, en: 

         en(j)=en(j)+switch(j)*v0*poly(j)

c  derivatives of switching function wrt the bondlegths r:

         do 1200 i=1,3
           dswdr(j,i)=-0.5d0*gamma(i)/
     &                         (dcosh(0.5d0*gamma(i)*rho(j,i)))**2
     &               *switch(j)/(1.0d0-dtanh(0.5d0*gamma(i)*rho(j,i)))

c  and the overall bondlegth derivatives, dvr:

           dvr(j,i)=dvr(j,i)
     &                 +dswdr(j,i)*v0*poly(j)
     &                 +switch(j)*v0*dpdr(j,i)

1200   continue

c  and that's it!!

       return
       end

c*******************************************************************

       subroutine outp(j)

c  subroutine tests energy conservation (or lack therof) throughout
c  each trajectory

c  subroutine performs end tests on the trajectories to test for reaction
c  or otherwise

c  subroutine calculates final relative velocity vector and 
c  angular momenta of the product fragments

       include 'traj.inc'

       common /outp1/ etot(nmax),ekin(nmax)

c  first check on the total energy etot
c  we need the potential energy en
c  and the kinetic energy enkin of the system

      ekin(j)=0.d0
      ntest(j)=1

      do 200 i=1,natom
          ekin(j)=ekin(j)+(v(j,i,1)**2+v(j,i,2)**2+
     &          v(j,i,3)**2)*0.5d0*amas(i)
200   continue

c   calculate the total energy above reference energy

        etot(j)=ekin(j)+en(j)

c   initialise headers for output files and output variables

      if(nloop.eq.0)then
        if(j.eq.1)then
 	  write(11,*)
	  call lisym(11,'-')
	  write(11,*)' **** Trajectory output information ****'
	  write(11,*)
          if(nmol.lt.4)then
	    write(11,*)
	    call lisym(11,'-')
            write(11,11)
11          format(2x,'j,  time (ps),     etot(j),'
     & ,'    rel. error,   r(HC),   r(CN),   r(HN)',/)
          endif
          avetot=0.0d0
          averr=0.0d0
          avek=0.0d0
          aven=0.0d0
        endif
	errmax=0.0d0
        errck(j)=0.0d0
        echeck(j) = etot(j)
        timf(j)=0.0d0
        nfin(j)=0
        nchannel(j)=0
      endif

c  next perform end checks on the configuration

c  following McCormack and Lim etc 
c  we choose to define reaction as occurring when the line joining 
c  the C-N centre-of-mass with the H atom makes an angle with 
c  the C-N line of centres that is greater than or equal to 113.1 degrees.

c  if not the trajectory is still going and we bypass the end tests
c  NB we are ignoring the possibility of fragmentation (ie consider
c  energies less than about 5.5 eV)

c  calculate the CN centre of mass

       xcm=(c(j,2,1)*amas(2)+c(j,3,1)*amas(3))/(amas(2)+amas(3))
       ycm=(c(j,2,2)*amas(2)+c(j,3,2)*amas(3))/(amas(2)+amas(3))
       zcm=(c(j,2,3)*amas(2)+c(j,3,3)*amas(3))/(amas(2)+amas(3))

c  calculate distance from H to CN centre-of-mass and from
c  CN centre-of-mass to the N atom:

       rcmh=(c(j,1,1)-xcm)**2+(c(j,1,2)-ycm)**2+(c(j,1,3)-zcm)**2
       rcmh=dsqrt(rcmh)

       rcmn=(c(j,3,1)-xcm)**2+(c(j,3,2)-ycm)**2+(c(j,3,3)-zcm)**2
       rcmn=dsqrt(rcmn)

c  calculate cos of the angle:

       ang=(xcm-c(j,1,1))*(c(j,3,1)-xcm)+
     &     (ycm-c(j,1,2))*(c(j,3,2)-ycm)+
     &     (zcm-c(j,1,3))*(c(j,3,3)-zcm)

       ang=-ang/(rcmh*rcmn)

c  calculate angle:

       ang=dacos(ang)
       refang=113.1d0*4.0d0*datan(1.0d0)/180.0d0

c  check to see whether reaction has occurred
c  where nchannel = 1 defines reaction

       if(ang.ge.refang)then
         ntest(j)=0
         nchannel(j)=1

c  calculate final state properties only as the end test is being 
c  first triggered

         if(nfin(j).eq.0)then
           nfin(j)=1

c  save final time for isomerization

           timf(j)=(nloop-1)*delta*tups

c  save average trajectory error statistics at the end of each trajectory

           avetot=avetot+etot(j)/nmol
           averr=averr+errck(j)/nmol
           avek=avek+ekin(j)/nmol
           aven=aven+en(j)/nmol
	 endif
       endif

c   calculate current time t in ps

       timps=(nloop-1)*delta*tups

c  check for the conservation of energy during trajectory

       if (nloop.gt.0) then       
         nbad = 0
           if(echeck(j).ne.0.0d0)then
             errck(j) = abs((echeck(j)-etot(j))/echeck(j))
           else
             errck(j)=-abs((echeck(j)-etot(j)))
           endif

c  if the trajectory is bad then stop, updating the error 
c  statistics

          if (errck(j).ge.1.0d-03)then
            nfin(j)=2
	    ntest(j)=0
            nbad = nbad + 1
            avetot=avetot+etot(j)/nmol
            averr=averr+errck(j)/nmol
            avek=avek+ekin(j)/nmol
            aven=aven+en(j)/nmol
          endif
        nbadm=max0(nbadm,nbad)
      else
        nbad=0
      endif

c  write out energy information

      if(nmol.lt.4.and.nloop.gt.0)then
          write(11,1) j,timps,etot(j),
     &    errck(j),r(j,1),r(j,2),r(j,3)
1         format(1x,i2,2x,e11.4,2(2x,e12.4),3(2x,f7.4))
          errmax=max(errmax,errck(j))
          if(nfin(j).ne.0)then
            write(11,*)
            if(j.eq.nmol)call lisym(11,'-')
          endif
      endif

      nstop=1
      nstop=nstop*nfin(j)

c  if we have reached the end of the allocated timesteps
c  update the trajectory statistics:

      if(nloop.eq.nstep)then
        avetot=avetot+etot(j)/nmol
        averr=averr+errck(j)/nmol
        avek=avek+ekin(j)/nmol
        aven=aven+en(j)/nmol
        if(nmol.lt.6)then
          write(11,*)
          if(j.eq.nmol)call lisym(11,'-')
        endif
      endif

      return
      end

c  ******************************************************************

       subroutine gwrite

c  subroutine outputs information from each trajectory batch

       include 'traj.inc'

c  write out overall energy statistics

       write(11,*)
       write(11,*)'   Trajectory statistics:'
       write(11,*)'   Averages after trajectory completion'
       write(11,*)
       call lisym(11,'-')
       write(11,*)'   <etot>      <rel.err.>      < k.e.>       <p.e.>'
       write(11,2) avetot,averr,avek,aven
2      format(1x,2g14.7,2x,g14.7,1x,g14.7)
       call lisym(11,'-')

c  calculate the total number of reactive trajectories 

       nreact=0
       do j=1,nmol
         if(nchannel(j).eq.1)nreact=nreact+1
       enddo

c  calculate probability of reaction over the "good" trajectories

       preact=0.0d0
       stderr=0.0d0

       if((nmol-nbadm).gt.0)then
	 tot=float(nmol-nbadm)
	 react=float(nreact)
         preact=react/tot
	 if(nreact.gt.0)then
	   stderr=preact*sqrt((tot-react)/(tot*react))
         endif
       endif

c  write out reaction probability

       write(11,*)
       write(11,*)'   Reaction Probability:'
       write(11,*)
       call lisym(11,'-')
       write(11,20)
       write(11,30)nmol,nbadm,nreact,preact,stderr
       call lisym(11,'-')
       write(11,*)

20     format(1x,'ntraj',2x,'nbad',2x,'nreact',2x,'Reaction Prob.',
     &        2x,'Standard Error')
30     format(i4,3x,i4,3x,i4,3x,g14.7,2x,g14.7)

c  write out distribution of final times for reaction 

       write(11,*)
       write(11,*)'Distribution of times for reaction:'
       write(11,*)
       call lisym(11,'-')
       write(11,*)'Time (ps), fract. trajectories unreacted:'

       ntraj=nmol-nbadm
       
       tbox=0.0d0
       unreact=float(nmol-nbadm)/float(ntraj)
       write(11,40)tbox,unreact

       do 100 n=1,40
         tbox=n*0.05d0
         if(tbox.gt.nstep*delta*tups)goto 200
         ntreact=0
         do j=1,nmol
           if(timf(j).le.tbox)then
             if(timf(j).ne.0.0d0)ntreact=ntreact+1
           endif
         enddo
         unreact=float(ntraj-ntreact)/float(ntraj)
         write(11,40)tbox,unreact
100    continue
       call lisym(11,'-')
        
200    continue

c      do j=1,nmol
c        write(11,40)timf(j)
c      enddo

40     format(2x,f7.4,2x,f7.4)


       return
       end
