
c-----------------------------------------------------------------------
c   emspac: 930910: H. W. Schranz
c   suite of EMS sampling programs
c
c   based on ejzmmi.f: 930730: H. W. Schranz
c   based on emsmmi.f: 901017: H. W. Schranz
c   based on emsmcl.f: 901017: H. W. Schranz
c   based on emsmkv.f: 891115: H. W. Schranz
c
c-----------------------------------------------------------------------
c   UPDATES:
c   --------
c
c
c scalar sequential form of Markov sampling for trajectory initialisation
c
c   #   ems method   state
c   1   ems          (E)
c   2   ejz          (E,J=0)      }
c   3   ejm          (E,J)        }  ejs methods
c   4   ejv          (E,J,Jz)     }
c
c 1. Set nstate=nmol and nmol=1 temporarily while generating initial states
c 2. Use array locations c(1,i,k) and cold(1,i,k), i=1,natom, k=1,3,
c  to store the configuration of the current and old state in the walk.
c 3. Use schatz, intern, ... angmom, etc...
c 4. Generated initial states are put in temporary storage arrays,
c 5. After all states are generated we reset nmol=nstate  and transfer 
c  configuration and velocities to c and v arrays
c
c-----------------------------------------------------------------------
      subroutine emspar

c      initialisation of ems parameters: read from EMS.PAR

      include 'traj.inc'
      include 'ems.inc'

c     call lisym(11,'-')
c     write(11,*) ' *** emspar: initialise EMS parameters'
c     write(11,*) ' Fragment energies: efraga=',efraga,' efragb=',efragb

c   read in from EMS.PAR on unit 7

c     open(unit=7,file='EMS.PAR',status='old')

c  hardwire these parameters

      nmksa1=1000
      nmksa2=1000
      dmksa=0.0150d0
      alphaa=0.0d0
      
      nmksb1=100
      nmksb2=100
      dmksb=1.5000d0
      alphab=0.0d0

      rrps(1)=2.50d0
      rrps(2)=2.50d0
      rrps(3)=2.50d0

c     read(7,80)comlin
c     write(11,81)comlin
   80 format(a75)
   81 format(1x,a75)
c     read(7,80)comlin
c     write(11,81)comlin
c     read(7,*)nmksa1,nmksa2,dmksa,alphaa
c     write(11,*)nmksa1,nmksa2,dmksa,alphaa
c     read(7,80)comlin
c     write(11,81)comlin
c     read(7,*)nmksb1,nmksb2,dmksb,alphab
c     write(11,*)nmksb1,nmksb2,dmksb,alphab
c     read(7,80)comlin
c     write(11,81)comlin
c     read(7,*)(rrps(i),i=1,nbond)
c     write(11,*)(rrps(i),i=1,nbond)
c     close(unit=7)

      do 300 i=1,3
        do 300 j=1,3
          rintol(i,j)=0.0d0
          rintns(i,j)=0.0d0
  300 continue
c
      dmi=0.0d0
c
      return
      end

      subroutine iniems(nfrag,ifrag,nbfrag,ibfrag,efrag)

      include 'traj.inc'
      include 'ems.inc'

      dimension ifrag(natom),ibfrag(nbond)

c     call lisym(11,'-')
c     write(11,*) ' *** iniems: initialise first Markov state'

c   set up initial state

      do 100 i=1,natom
        c(1,i,1)=tc(1,i,1)
        c(1,i,2)=tc(1,i,2)
        c(1,i,3)=tc(1,i,3)
  100 continue

c     write(11,*)
c     write(11,*)' initial cartesian coords.: tc(1,i,k):'
c     write(11,150)(i,(tc(1,i,k),k=1,3),i=1,natom)
  150 format(1x,'atom',i6,' coords.:',3g13.5)

c   get P.E. of initial state and use as reference energy
c   efrag= fragment energy (relative to enminim)
c   eintst= efrag
c   enminim=reference P.E. for configuration tc(1,i,k)
c   epenmin=energy in absolute scale of schatz

c  reset values of PES constants

      call intern(1)
      call dvdi(1)
      eintst=efrag
      enminim=en(1)
      epenmin=eintst + enminim

c   get weight of initial state

      call weight2(nfrag,ifrag,nbfrag,ibfrag)

      write(11,*)
      write(11,1)eintst,enunit
 1    format(/,'   eintst = total internal energy=',g12.5,a10)
c     write(11,*) ' Total Internal Energy=',efrag,' eintst=',eintst
c     write(11,*) ' enminim=reference P.E. for configuration tc(1,i,k)'
c     write(11,*) '        =en(1)=',enminim
c     write(11,*) ' epenmin=energy in absolute scale of schatz=',epenmin
c     write(11,*) ' statistical weight of first state: wt=',wt,' wi=',wi

      return
      end

      subroutine zstmkv
c
c   zero statistics counters for markov chain
c
      include 'traj.inc'
      include 'ems.inc'
c
      nrej=0
      wratio=0
      totwt=0.0d0
      ensum=0.0d0
      do 4400 i=1,nbond
        rsum(i)=0.0d0
 4400 continue
      enmin=en(1)
      enmax=en(1)
      ekmin=ek
      ekmax=ek
      return
      end

      subroutine istmkv
c
c   increment statistics counters for markov chain
c
      include 'traj.inc'
      include 'ems.inc'
c
c
      totwt=totwt+wi
      ensum=ensum+en(1)*wi
      do 100 i=1,nbond
        rsum(i)=rsum(i)+r(1,i)*wi
  100 continue
      if(en(1).lt.enmin)enmin=en(1)
      if(en(1).gt.enmax)enmax=en(1)
      if(ek.lt.ekmin)ekmin=ek
      if(ek.gt.ekmax)ekmax=ek
c      write(11,200)en(1),enmin,enmax,ek,ekmin,ekmax
c  200 format(' en=',g12.5,'(',g12.5,g12.5,') ek=',
c     +  g12.5,'(',g12.5,g12.5,')') 
      return
      end

      subroutine stmkv
c
c   routine which echos statistics about markov sampling
c
      include 'traj.inc'
      include 'ems.inc'
c
      ntostp=nstate*nmkst2
      if(ntostp.gt.0)then
        rnrnt=dfloat(nrej)/dfloat(ntostp)
        wrat1=wratio/dfloat(ntostp)
c       write(11,2325)ntostp,nrej,rnrnt,en(1),enunit,ek,enunit
 2325   format(/,'   Markov chain sampling statistics',
     &  /,'   after',i10,'  markov steps:  nrej=',i10,
     &  /,'   rnrnt=rejections/total =',g12.5,
     &  /,'   Characteristics of last state selected:',
     &  /,'   en(1)=potential energy=',g15.8,a10,
     &  /,'   ek=kinetic energy =',g12.5,a10)
c       write(11,2325)ntostp,nrej,rnrnt,wrat1,en(1),enunit,
c    +  ek,enunit,wt,wi
c2325   format(/,' Markov chain sampling statistics',
c    +  /,' after',i10,'  markov steps:  nrej=',i10, 
c    1  /,' rnrnt=rejections/total =',g12.5,
c    +  /,' wrat1=successes/total  =',g12.5,
c    +  /,' Characteristics of last state selected:',
c    1  /,' en(1)=potential energy=',g12.5,a10,
c    1  /,' ek=kinetic energy =',g12.5,a10,
c    +  /,' wt=weight=',g12.5,
c    +  /,' wi=importance weight=',g12.5,
c    +  /,' (c(1,i,k),k=1,3):')
c       write(11,3700)((c(1,i,k),k=1,3),i=1,natom)
c3700   format(1x,3g13.5)
 4330   continue
c
c  check averages and max/min values of en, ek, r(1,i)
c
        if(totwt.le.0.0d0)then
          write(11,*)' totwt=',totwt
          stop
        endif
c
        if(eintst.le.0.0d0)then
          write(11,*)' eintst=',eintst
          stop
        endif
c
        ensum=(ensum/totwt)-enminim
        enovere=ensum/eintst
        do 4500 i=1,nbond
          rsum(i)=rsum(i)/totwt
 4500   continue
c       write(11,5200)eintst,enunit,enminim,enunit,epenmin,enunit,
c    +    totwt,ensum,enunit,enovere
 5200   format(/,' *** CHECK MARKOV CHAIN AVERAGES ***',
     +  /,' eintst = total internal energy=',g12.5,a10,
     +  /,' enminim = assumed minimum potential energy=',g12.5,
     +  a10,
     +  /,' epenmin = eintst + enminim=',g12.5,a10,
     +  /,' total weight of all markov states =',g12.5,
     +  /,' <V>    = average potential energy=',g12.5,a10,
     +  ' (above energy minimum)',
     +  /,' <V>/E  = average potential energy/total energy=',g12.5)
 
c       write(11,5300)
c    +    enmin,enunit,enmax,enunit,ekmin,enunit,ekmax,enunit
 5300   format(
     +  /,' enmin =',g17.10,a10,' enmax =',g17.10,a10,
     +  /,' ekmin =',g17.10,a10,' ekmax =',g17.10,a10)
c       write(11,*)' Average bondlengths: (<r(1,i)>, i=1,...,nbond):'
c       write(11,5400)(rsum(i),i=1,nbond)
 5400   format(1x,5g12.5)
c       call lisym(11,'-')
      endif
      return
      end

      subroutine weight2(nfrag,ifrag,nbfrag,ibfrag)
c
c   calculate the appropriate weight wt.
c
c
      include 'traj.inc'
      include 'ems.inc'

      dimension ifrag(natom),ibfrag(nbond)
c
c
c   calculate potential and kinetic energy
c
      call dvdi(1)
      ek=epenmin-en(1)
      if(ek.gt.0.0d0)then
c
c   subtract centre of mass from position coordinates,
c   calculate moment of inertia tensor around centre of mass,
c   and calculate product of the principal moments of inertia.
c
        call cmzerc(nfrag,ifrag,tamas)

c   two cases

        if(nfrag.eq.2)then

c   diatomic case  nfrag=2

c   calculate diatomic moment of inertia & weight for diatomic fragment

          dmi=rdmd*r(1,ibfrag(1))**2
          wi=ek**alpha
          wt=1.0d0/(dsqrt(ek)*dmi*wi)
        else

c  polyatomic case  nfrag>2

          call intens(1,nmax,natom,nfrag,ifrag,c,amas,rintns,0)
          call ppmcal(rintns,ppmi,0)
          if(ppmi.le.0.0d0)then
c           write(11,100)ppmi
  100       format(/,' *** ABORT from routine weight2: ppmi=',g12.5)
          endif
          wi=ek**alpha
          wt=(dsqrt(ek)**(3*nfrag-8))/(dsqrt(ppmi)*wi)
        endif
      else
        wi=1.0d0
        wt=0.d0
      endif

      return
      end

      subroutine mstep2(nfrag,ifrag,nbfrag,ibfrag)
c
c   performs a markov step in the configuration space (x_1, y_i ,z_i)
c   of the reactant molecule
c   by picking a atom and changing its position coordinates 
c   by up to +/- dstxyz/2.
c   the trial state is accepted with a
c   probability wt/wtold.
c
c
      include 'traj.inc'
      include 'ems.inc'

      dimension ifrag(natom),ibfrag(nbond)
c
c
c*****design csave and crest to be the same for all EMS variants???
c
c   save old position coordinates of atoms
c
      call csave
c
c   all fragment atoms are moved by random changes to each of
c   their xyz components of up to +/- dstxyz/2.
c
      do 1000 i=1,nfrag
        c(1,ifrag(i),1)=c(1,ifrag(i),1)+(usran(ir)-0.5d0)*dstxyz
        c(1,ifrag(i),2)=c(1,ifrag(i),2)+(usran(ir)-0.5d0)*dstxyz
        c(1,ifrag(i),3)=c(1,ifrag(i),3)+(usran(ir)-0.5d0)*dstxyz
 1000 continue
c
c   check if trial state is outside reactant space
c
      call chksta(nbfrag,ibfrag,nflag)
c
c   calculate weight of current state if nflag=0
c
      if(nflag.eq.0)then
        call weight2(nfrag,ifrag,nbfrag,ibfrag)
c        write(11,*)' weight2:  en(1)=',en(1),' wt=',wt,' wi=',wi
c        write(11,*)' r(1,k)=',(r(1,k),k=1,nbond)
      else
c   then this trial state has zero weight and the old state will be used
        nrej=nrej+1
        call crest
c        write(11,*) ' mstep2: nflag.ne.0: reject trial state'
        return
      endif
c
c   calculate <wratio>=average(wt/wtold)
c                     =sum(wow)
c      wow=1       wt>wtold
c         =wt/wtold wt<wtold
c         =0       for r outside range or vr outside range (wt=0)
c              (note that this last case is automatically excluded)
c
      if(wt.gt.wtold)then
c
c   accept trial state
c   (trial state will saved as old state on next pass through routine mstep)
c
        wratio=wratio+1.0d0
c        write(11,*) ' mstep2: wt.gt.wtold: accept trial state'
        return
      else
        wow=wt/wtold
        wratio=wratio+wow
        if(wow.gt.usran(ir))then
c
c   accept trial state
c   (trial state will saved as old state on next pass through routine mstep)
c
c          write(11,*) ' mstep2: wow.gt.usran(ir): accept trial state'
          return
        else
c
c   reject trial state => accept old state
c
          nrej=nrej+1
          call crest
c          write(11,*) ' mstep2: low weight: reject trial state'
          return
        endif
      endif
      end

      subroutine csave
c
c   save current markov state
c
c
      include 'traj.inc'
      include 'ems.inc'
c
c
      enold=en(1)
      ekold=ek
      wtold=wt
      wiold=wi
c
      do 100 i=1,natom
        cold(1,i,1)=c(1,i,1)
        cold(1,i,2)=c(1,i,2)
        cold(1,i,3)=c(1,i,3)
  100 continue
c
      do 300 i=1,3
        do 300 j=1,3
          rintol(i,j)=rintns(i,j)
  300 continue
c
      dmiold=dmi
c
      return
      end

      subroutine crest
c
c   restore previous markov state
c
c
      include 'traj.inc'
      include 'ems.inc'
c
c
      en(1)=enold
      ek=ekold
      wt=wtold
      wi=wiold
c
      do 100 i=1,natom
        c(1,i,1)=cold(1,i,1)
        c(1,i,2)=cold(1,i,2)
        c(1,i,3)=cold(1,i,3)
  100 continue
c
      do 300 i=1,3
        do 300 j=1,3
          rintns(i,j)=rintol(i,j)
  300 continue
c
      dmi=dmiold
c
      return
      end

      subroutine chksta(nbfrag,ibfrag,nflag)
c
c   routine for checking whether current configuration is
c   in reactant phase space.
c   if yes we set nflag=0
c   if no  we set nflag=1
c
c
      include 'traj.inc'
      include 'ems.inc'
      dimension ibfrag(nbond)
c
c
      nflag=1

      call intern(1)

c   check bondlengths in fragment

      do 100 ibf=1,nbfrag
        if(r(1,ibfrag(ibf)).gt.rrps(ibfrag(ibf)))return
  100 continue

c   in principle we could check bond angles  and torsion angles

c   set flag to zero as we do have a reactant state!

      nflag=0

      return
      end

      subroutine zstmom
c
c      routine to initialise values of momenta statistics
c
c
      include 'traj.inc'
      include 'ems.inc'
c
c   initialise statistical counters
c
      do 600 i=1,natom
        vsum(i,1)=0.0d0
        vsum(i,2)=0.0d0
        vsum(i,3)=0.0d0
        v2sum(i,1)=0.0d0
        v2sum(i,2)=0.0d0
        v2sum(i,3)=0.0d0
  600 continue
      nvsel=0
c
      amsum=0.0d0
      am2sum=0.0d0
      ersum=0.0d0
c
      return
      end

      subroutine stmom(nfrag,ifrag)
c
c      routine to output values of momenta statistics
c
c
      include 'traj.inc'
      include 'ems.inc'

      dimension ifrag(natom)

c
      if(nvsel.le.0)return
c     write(11,100)nvsel
  100 format(
     +/,' *** STATISTICS OF MONTE CARLO SELECTED VELOCITIES ***',
     +/,' total number of selected velocities=',i10)

c     write(11,*) 
c     write(11,*) ' This fragment has ',nfrag,' atoms numbered as:'
c     write(11,*) (ifrag(i),i=1,nfrag)
c     write(11,*) 

      vcmx=0.0d0
      vcmy=0.0d0
      vcmz=0.0d0
      avke=0.0d0
      do 600 i=1,nfrag
        vsum(ifrag(i),1)=vsum(ifrag(i),1)/dfloat(nvsel)
        vsum(ifrag(i),2)=vsum(ifrag(i),2)/dfloat(nvsel)
        vsum(ifrag(i),3)=vsum(ifrag(i),3)/dfloat(nvsel)
        v2sum(ifrag(i),1)=v2sum(ifrag(i),1)/dfloat(nvsel)
        v2sum(ifrag(i),2)=v2sum(ifrag(i),2)/dfloat(nvsel)
        v2sum(ifrag(i),3)=v2sum(ifrag(i),3)/dfloat(nvsel)
        vcmx=vcmx+amas(ifrag(i))*vsum(ifrag(i),1)
        vcmy=vcmy+amas(ifrag(i))*vsum(ifrag(i),2)
        vcmz=vcmz+amas(ifrag(i))*vsum(ifrag(i),3)
        avke=avke+amas(ifrag(i))*(v2sum(ifrag(i),1)
     .    +v2sum(ifrag(i),2)+v2sum(ifrag(i),3))
  600 continue
      vcmx=vcmx/tamas
      vcmy=vcmy/tamas
      vcmz=vcmz/tamas
      avke=avke/2.0d0
c     write(11,700)
  700 format(/,' <vx_i>, <vy_i>, <vz_i>=',
     +' average velocities of each atom:')
c     write(11,750)(vsum(ifrag(i),1),vsum(ifrag(i),2),
c    .  vsum(ifrag(i),3),i=1,nfrag)
  750 format(2x,6g12.5)
c     write(11,*)' <vxCM>=',vcmx
c     write(11,*)' <vyCM>=',vcmy
c     write(11,*)' <vzCM>=',vcmz
c     write(11,800)
  800 format(/,' <vx_i**2>, <vy_i**2>, <vz_i**2>=',
     +' average squared velocities of each atom:')
c     write(11,750)(v2sum(ifrag(i),1),v2sum(ifrag(i),2),
c    .  v2sum(ifrag(i),3),i=1,nfrag)
c     write(11,900)avke,enunit
  900 format(/,' Average kinetic energy in all selected states=',
     +g12.5,a10)
c
      amsum=amsum/dfloat(nvsel)
      am2sum=am2sum/dfloat(nvsel)
      ersum=ersum/dfloat(nvsel)
c     write(11,1000)amsum,am2sum,ersum,enunit
 1000 format(
     +/,' <J>    = average angular momentum           =',g12.5,
     +/,' <J**2> = average squared angular momentum   =',g12.5,
     +/,' <EJ>   = average rotational energy          =',g12.5,
     +a10)
c
      return
      end

      subroutine momgen2(nfrag,ifrag)
c
c      routine to initialise values of momenta (velocities)
c      uniformly according to the microcanonical
c      ensemble.
c
c
c  generation of random numbers on normal distribution:
c  p(y)dy=exp(-y*y/2)dy/sqrt(2pi)
c
      include 'traj.inc'
      include 'ems.inc'

      dimension vrot(natom,3)
      dimension ifrag(natom)

c
c   generate internal coordinates from cartesians as these may not
c   be up to date if the state we are using was a restored state.
c      call intern(1)
c   debug output:
c      write(11,*)
c      write(11,*) ' momgen2'
c      write(11,*) ' totwt=',totwt
c      write(11,*) ' en(1)=',en(1),' ek=',ek,' wt=',wt,' dmi=',dmi
c      write(11,*) ' enold=',enold,' ekold=',ekold,' wtold=',wtold,
c     .  ' dmiold=',dmiold
c      write(11,*)' routine momgen2: current bondlengths r'
c      write(11,*)(r(1,k),k=1,nbond)
c      write(11,*)' routine momgen2: current coordinates x, y, z'
c      do 155 i=1,nfrag
c        write(11,*)(c(1,ifrag(i),k),k=1,3)
c  155 continue
c      write(11,*)' routine momgen2: old coordinates x, y, z'
c      do 156 i=1,nfrag
c        write(11,*)(cold(1,ifrag(i),k),k=1,3)
c  156 continue
c
c 890526:to generate the unscaled velocities we need to weight
c   the normally distributed random numbers gasdev(ir)
c   by 1/sqrt(amas(i)) where i=1,nfrag, ixyz=1,2,3,
c   because heavier atoms move more slowly,
c   even though they have more momentum than lighter atoms.
c   (v proportional to 1/sqrt(m) but p proportional to sqrt(m) for same energy)
c   f(p)=exp(-p*p/2mkT)=exp(-m*v*v/2kT), p=mv
c   
      do 350 i=1,nfrag
        dsqrms=dsqrt(amas(ifrag(i)))
        v(1,ifrag(i),1)=gasdev(ir)/dsqrms
        v(1,ifrag(i),2)=gasdev(ir)/dsqrms
        v(1,ifrag(i),3)=gasdev(ir)/dsqrms
  350 continue
c   debug output:
c      write(11,*)' routine momgen2: unscaled velocities vx, vy, vz'
c      do 355 i=1,nfrag
c        write(11,*)(v(1,ifrag(i),k),k=1,3)
c  355 continue
c
c   calculate the unscaled centre of mass velocity and subtract this
c   off the unscaled velocities to yield the unscaled velocities
c   relative to the centre of mass
c
      call cmzerv(nfrag,ifrag,tamas)
c   debug output:
c      write(11,*)' routine momgen2: cm-unscaled velocities vx, vy, vz'
c      do 356 i=1,nfrag
c        write(11,*)(v(1,ifrag(i),k),k=1,3)
c  356 continue
c
c   subtract off spurious angular momentum
c   we assume that the position coordinates (x,y,z) have been
c   moved to the centre of mass frame. This should occur in routine weight.
c
      call angmom(1,nfrag,ifrag)
c debug output:
c      write(11,*)' J =',ammag
c      write(11,*)' Jvec=',(amvec(k),k=1,3)
c
      if(nfrag.eq.2)then
c   diatomic angular velocity nfrag=2
        wang(1)=amvec(1)/dmi
        wang(2)=amvec(2)/dmi
        wang(3)=amvec(3)/dmi
      else
c   polyatomic angular velocity nfrag>2
        call angvel(0)
      endif
c
      call velrot(1,nfrag,ifrag,vrot,0)
c
      do 450 i=1,nfrag
        v(1,ifrag(i),1)=v(1,ifrag(i),1)-vrot(ifrag(i),1)
        v(1,ifrag(i),2)=v(1,ifrag(i),2)-vrot(ifrag(i),2)
        v(1,ifrag(i),3)=v(1,ifrag(i),3)-vrot(ifrag(i),3)
  450 continue
c   debug output:
c      write(11,*)' routine momgen2: nr-unscaled velocities vx, vy, vz'
c      do 455 i=1,nfrag
c        write(11,*)(v(1,ifrag(i),k),k=1,3)
c  455 continue
c
c   calculate unscaled internal kinetic energy
c
      ekun=0.d0
      do 500 i=1,nfrag
        ekun=ekun+amas(ifrag(i))*(v(1,ifrag(i),1)**2
     .    + v(1,ifrag(i),2)**2 + v(1,ifrag(i),3)**2)
  500 continue
      ekun=ekun/2.d0
c
c   calculate scale factor
c
      scale=dsqrt(ek/ekun)
c
c   scale velocity components
c
      do 600 i=1,nfrag
        v(1,ifrag(i),1)=v(1,ifrag(i),1)*scale
        v(1,ifrag(i),2)=v(1,ifrag(i),2)*scale
        v(1,ifrag(i),3)=v(1,ifrag(i),3)*scale
c***calculate statistics on final velocities
        vsum(ifrag(i),1)=vsum(ifrag(i),1)+v(1,ifrag(i),1)
        vsum(ifrag(i),2)=vsum(ifrag(i),2)+v(1,ifrag(i),2)
        vsum(ifrag(i),3)=vsum(ifrag(i),3)+v(1,ifrag(i),3)
        v2sum(ifrag(i),1)=v2sum(ifrag(i),1)
     .    +v(1,ifrag(i),1)*v(1,ifrag(i),1)
        v2sum(ifrag(i),2)=v2sum(ifrag(i),2)
     .    +v(1,ifrag(i),2)*v(1,ifrag(i),2)
        v2sum(ifrag(i),3)=v2sum(ifrag(i),3)
     .    +v(1,ifrag(i),3)*v(1,ifrag(i),3)
  600 continue
c   debug output:
c      write(11,*)' routine momgen2: scaled velocities vx, vy, vz'
c      do 655 i=1,nfrag
c        write(11,*)(v(1,ifrag(i),k),k=1,3)
c  655 continue
      nvsel=nvsel+1
c***calculate angular momentum for final (positions,velocities)
      call angmom(1,nfrag,ifrag)
c debug output:
c      write(11,*)' J =',ammag
c      write(11,*)' Jvec=',(amvec(k),k=1,3)
      amsum=amsum+ammag
      am2sum=am2sum+ammag**2
c
      if(nfrag.eq.2)then
c   diatomic angular velocity nfrag=2
        wang(1)=amvec(1)/dmi
        wang(2)=amvec(2)/dmi
        wang(3)=amvec(3)/dmi
      else
c   polyatomic angular velocity nfrag>2
        call angvel(0)
      endif
c
      call roten(erot,0)
      ersum=ersum+erot
c
      return
      end

      subroutine ppmcal(rintns,ppmi,iprint)
c
c  calculate product of principal moments of inertia
c  from elements in the moment of inertia tensor
c       
      implicit real*8 (a-h,o-z)
      dimension rintns(3,3)
c
      ppmi=rintns(1,1)*rintns(2,2)*rintns(3,3)
     +     +2.0d0*rintns(1,2)*rintns(1,3)*rintns(2,3)
     +     -rintns(1,1)*rintns(2,3)**2
     +     -rintns(2,2)*rintns(1,3)**2
     +     -rintns(3,3)*rintns(1,2)**2
c
      if(iprint.gt.0)then
c       write(11,100)ppmi
  100   format(/,' routine ppmcal:',
     +  ' product of principal moments of inertia =',g18.10e3)
      endif
      return
      end

