
      subroutine init
c--------------------------------------------------------------------------- 
c
c   generate initial states for the trajectory 
c
      include 'traj.inc'
      include 'ems.inc'
 
c   section of code for generating initial states of polyatomic fragments
c   via an EMS sampling procedure.

c   EMS sampling is done for each fragment A (unless an atom)(nfraga=1)

      call lisym(11,'-')
      write(11,*)' *** subroutine init: trajectory initial conditions'

      if(nfraga.gt.1)then
        if(efraga.gt.0.0d0)then
          call emscal(nfraga,ifraga,nbfraga,ibfraga,efraga,1)
        else
c   efraga=0 : set up molecule in equilibrium position with zero velocities
          call zeren(nfraga,ifraga)
        endif
      else

c   nfraga=1 : Fragment A is an atom
c   move centre of mass position and velocity of atom to origin

        call zerat(nfraga,ifraga)

      endif

c  for a unimolecular reaction there is only a single fragment,
c  reference to fragment B, orientation, impact parameter etc
c  have been removed

c   save position and velocity cinit,vinit into arrays c,v

      do 300 i=1,natom
        do 250 j=1,nmol
          c(j,i,1)=cinit(j,i,1)
          c(j,i,2)=cinit(j,i,2)
          c(j,i,3)=cinit(j,i,3)
  250 continue
  300 continue

      do 400 i=1,natom
        do 350 j=1,nmol
          v(j,i,1)=vinit(j,i,1)
          v(j,i,2)=vinit(j,i,2)
          v(j,i,3)=vinit(j,i,3)
  350 continue
  400 continue

c   arrays c and v should now contain the coordinates and
c   velocities in their respective centre of mass frames.

c   store initial positions & velocities

      do 500 i=1,natom
        do 450 j=1,nmol
          cinit(j,i,1)=c(j,i,1)
          cinit(j,i,2)=c(j,i,2)
          cinit(j,i,3)=c(j,i,3)
  450 continue
  500 continue

      do 600 i=1,natom
        do 550 j=1,nmol
          vinit(j,i,1)=v(j,i,1)
          vinit(j,i,2)=v(j,i,2)
          vinit(j,i,3)=v(j,i,3)
  550 continue
  600 continue

c   write initial conditions out to file WQV

c     call writqv

c--------------------------------------------------------------------------- 
c initialise further parameters
 
      nloop = 0
      nbadm = 0
 
      return
      end

      subroutine emscal(nfrag,ifrag,nbfrag,ibfrag,efrag,nwhich)
c
c   generate initial states of polyatomic fragments
c   via an EMS sampling procedure.
c
      include 'traj.inc'
      include 'ems.inc'

      dimension ifrag(natom),ibfrag(nbond)

c     write(11,*) 
c     write(11,*) ' *** ROUTINE EMSCAL *** '
c     write(11,*) 

c   set parameters consistent with scalar (sequential) mode of sampling

      nstate=nmol
      nmol=1

      if(nwhich.eq.1)then
        dstxyz=dmksa
        alpha=alphaa
        nmkst1=nmksa1
        nmkst2=nmksa2
c       write(11,*) ' Set EMS walk parameters for fragment A'
c       write(11,*) ' dstxyz=',dstxyz,' alpha=',alpha
c       write(11,*) ' nmkst1=',nmkst1,' nmkst2=',nmkst2
      elseif(nwhich.eq.2)then
        dstxyz=dmksb
        alpha=alphab
        nmkst1=nmksb1
        nmkst2=nmksb2
c       write(11,*) ' Set EMS walk parameters for fragment B'
c       write(11,*) ' dstxyz=',dstxyz,' alpha=',alpha
c       write(11,*) ' nmkst1=',nmkst1,' nmkst2=',nmkst2
      else
        write(11,*) ' ABORT: routine emscal:',
     +   ' nwhich must be 1 (fragment A) or 2 (fragmentB)' 
        stop
      endif

c   if nfrag=2 we calculate the reduced mass for the diatomic used
c   in routine weight2

      if(nfrag.eq.2)then
c       write(11,*) ' DIATOMIC FRAGMENT: nfrag=2'
     
        rdmd = 1.0d0/( 1.0d0/amas(ifrag(1)) + 1.0d0/amas(ifrag(2)) )

c       write(11,*) ' Masses of diatomic =',(amas(ifrag(i)),i=1,nfrag)
c       write(11,*) ' Reduced mass of diatomic fragment =',rdmd
      endif

c  generate total mass of fragment

      tamas=0.0d0
      do 100 i=1,nfrag
        tamas=tamas + amas(ifrag(i))
  100 continue

c  shift the reference coordinate frame so the fragment in question is at 
c  the origin (corrections made 6th April 1995 by MJTJ)

c     write(11,*) ' Initial coordinates of reference state:'
c     write(11,*)
      do 115 j=1,nmol
        ccom(j,1)=0.0d0
        ccom(j,2)=0.0d0
        ccom(j,3)=0.0d0
        do 105 i=1,nfrag
          ccom(j,1)=ccom(j,1)+amas(ifrag(i))*c(j,ifrag(i),1)
          ccom(j,2)=ccom(j,2)+amas(ifrag(i))*c(j,ifrag(i),2)
          ccom(j,3)=ccom(j,3)+amas(ifrag(i))*c(j,ifrag(i),3)
105     continue
        ccom(j,1)=ccom(j,1)/tamas
        ccom(j,2)=ccom(j,2)/tamas
        ccom(j,3)=ccom(j,3)/tamas

c  subtract fragment centre of mass from all reference coordinates to 
c  maintain initial fragment separation  ^^^

        do 110 n=1,natom
          tc(j,n,1)=tc(j,n,1)-ccom(j,1)
          tc(j,n,2)=tc(j,n,2)-ccom(j,2)
          tc(j,n,3)=tc(j,n,3)-ccom(j,3)
c         write(11,*)(tc(j,n,k),k=1,3)
110     continue 
115   continue

c     write(11,*) 
c     write(11,*) ' Total mass of fragment of ',nfrag,' atoms'
c     write(11,*) ' composed of atoms numbered'
c     write(11,*) (ifrag(i),i=1,nfrag)
c     write(11,*) ' with masses'
c     write(11,*) (amas(ifrag(i)),i=1,nfrag)
c     write(11,*) ' is ',tamas
c     write(11,*) 

c  generate initial state from that stored in tc(i,k)

      call iniems(nfrag,ifrag,nbfrag,ibfrag,efrag)

c  initialise Markov chain statistical counters
      
      call zstmkv
      call zstmom

c     write(11,*) ' Fragment energy: efrag=',efrag,' eintst=',eintst

c  loop over incubation of chain

      do 1000 imkst=1,nmkst1

c  Do a Markov step

        call mstep2(nfrag,ifrag,nbfrag,ibfrag)

 1000 continue

c   loop over selected states

      do 3000 istate=1,nstate

c   loop over generation spatial configurations q

        do 2000 imkst=1,nmkst2

c  Do a Markov step

          call mstep2(nfrag,ifrag,nbfrag,ibfrag)

 2000   continue

c   increment statistics of Markov walk

      call istmkv

c   generate momenta (velocities) that satisfy K=E-V(q)

        call momgen2(nfrag,ifrag)

c   save positions and velocities in arrays cinit & init

        do 2500 m=1,nfrag
          cinit(istate,ifrag(m),1)=c(1,ifrag(m),1)
          cinit(istate,ifrag(m),2)=c(1,ifrag(m),2)
          cinit(istate,ifrag(m),3)=c(1,ifrag(m),3)
          vinit(istate,ifrag(m),1)=v(1,ifrag(m),1)
          vinit(istate,ifrag(m),2)=v(1,ifrag(m),2)
          vinit(istate,ifrag(m),3)=v(1,ifrag(m),3)
 2500   continue 

 3000 continue

c   statistics of Markov walk

      call stmkv

      call stmom(nfrag,ifrag)

c   reset parameters consistent with vector mode of rest of program

      nmol=nstate

      return
      end

      subroutine zeren(nfrag,ifrag)

c set up molecule in equilibrium position with zero velocities

      include 'traj.inc'
      include 'ems.inc'

      dimension ifrag(natom)
 
      do 30 istate=1,nmol
        do 20 m=1,nfrag
          cinit(istate,ifrag(m),1)=tc(1,ifrag(m),1)
          cinit(istate,ifrag(m),2)=tc(1,ifrag(m),2)
          cinit(istate,ifrag(m),3)=tc(1,ifrag(m),3)
          vinit(istate,ifrag(m),1)=0.0d0
          vinit(istate,ifrag(m),2)=0.0d0
          vinit(istate,ifrag(m),3)=0.0d0
   20   continue 
   30 continue

      return
      end

      subroutine zerat(nfrag,ifrag)

c   For the fragment which is an atom
c   move centre of mass position and velocity of atom to origin

      include 'traj.inc'
      include 'ems.inc'

      dimension ifrag(natom)
 
      do 100 istate=1,nmol
        do 50 m=1,nfrag
          cinit(istate,ifrag(m),1)=0.0d0
          cinit(istate,ifrag(m),2)=0.0d0
          cinit(istate,ifrag(m),3)=0.0d0
          vinit(istate,ifrag(m),1)=0.0d0
          vinit(istate,ifrag(m),2)=0.0d0
          vinit(istate,ifrag(m),3)=0.0d0
   50   continue 
  100 continue

      return
      end

