
       program traj

c  this program runs trajectories over a given potential energy
c  surface as follows:

c  	*initial conditions are chosen for the trajectories via a Markov
c  	 walk through configuration space

c  	*a number of trajectories are run from these initial 
c        conditions using a velocity-verlet integration algorithm

c  the program will also calculate normal mode frequencies and rotational
c  constants for input cartesian geometries

c  control parameters are read in through the various subroutines:
c
c                         CNTL,READZ,READC,EMSPAR
c
c       *trajectory output file is test.out

c  Latest Update: 6th January 1998

      include 'traj.inc'
      include 'ems.inc'

      open (unit=11,file='hnc.out',status='unknown')

c   read in run control parameters

      call cntl

c   read in the input cartesian coordinates and atomic masses

      call readc

c   read in the " z-matrix " type of internal coordinate
c   definition and other information

      call readz

c   read in ems parameters

      call emspar

c  read in the constants defining the potential energy surface

       call constants

       write(6,*)
       write(6,*)' Program running...'

c  calculate energy for input geometry

       call intern(1)
       call dvdi(1)
       write(11,1)r(1,1),r(1,2),r(1,3),en(1),enunit
 1     format(/,'   C-H bondlength = ',f7.4,' Angstroms',
     &        /,'   C-N bondlength = ',f7.4,' Angstroms',
     &        /,'   H-N bondlength = ',f7.4,' Angstroms',/,   
     &        /,'   ENERGY FOR INPUT GEOMETRY = ',g15.8,a10)

c  calculate normal mode frequencies for the input geometry 

       call nmode(nfraga,ifraga,tamas,1)

c  calculate the rotational constants for the input geometry

       call bvalue(nfraga,ifraga,tamas)

c  select appropriate initial conditions

c  check that trajectories will be performed:

       if(nstep.le.1)then
         write(11,*)'  No trajectories calculated...'
         write(11,*)'  Normal successful completion'
         stop
       endif

c  work

       if(tc(1,1,1).eq.0.0d0.and.tc(1,2,1).eq.0.0d0)tc(1,1,1)=0.001d0
       if(tc(1,1,2).eq.0.0d0.and.tc(1,2,2).eq.0.0d0)tc(1,1,2)=0.001d0
c  distort geometry slightly so as to ensure intitial conditions
       call lisym(11,'-')
       write(11,*)' random seed for initial conditions:',ir
       call init

c  run trajectories

c ****  run each trajectory serially ****

       write(11,2)nmol,delta*tups,nstep,nstep*delta*tups,nprint
 2     format(/,' *** TRAJECTORY CALCULATIONS ***',/
     &       ,/,'     Total number of trajectories run = ',i6
     &       ,/,'     Timestep = ',f8.6,' ps'
     &       ,/,'     Maximum number of timesteps per trajectory = ',i6
     &       ,/,'     Maximum time per trajectory = ',f8.6,' ps'
     &       ,/,'     Trajectories analysed every ',i4,' timesteps')

       do 10 j=1,nmol
         call velverlet(j)
10     continue

c  analyse and write out relevant results

       call gwrite

c      read(11,*)
c      write(11,*)' ...next random number seed is:'
c      write(11,*) ir

       write(6,*) ' Normal successful completion.'
       close (unit=11)

       stop
       end

